@@ -45,8 +45,11 @@ static int fio_null_commit(struct thread_data *td)
 {
 	struct null_data *nd = td->io_ops->data;
 
-	nd->events += nd->queued;
-	nd->queued = 0;
+	if (!nd->events) {
+		nd->events = nd->queued;
+		nd->queued = 0;
+	}
+
 	return 0;
 }
 
@@ -56,6 +59,8 @@ static int fio_null_queue(struct thread_data fio_unused *td, struct io_u *io_u)
 
 	if (td->io_ops->flags & FIO_SYNCIO)
 		return FIO_Q_COMPLETED;
+	if (nd->events)
+		return FIO_Q_BUSY;
 
 	nd->io_us[nd->queued++] = io_u;
 	return FIO_Q_QUEUED;
@@ -66,18 +71,8 @@ static int fio_null_setup(struct thread_data *td)
 	struct fio_file *f;
 	unsigned int i;
 
-	if (!td->total_file_size) {
-		log_err("fio: need size= set\n");
-		return 1;
-	}
-
-	td->io_size = td->total_file_size;
-	td->total_io_size = td->io_size;
-
-	for_each_file(td, f, i) {
-		f->real_file_size = td->total_io_size / td->nr_files;
-		f->file_size = f->real_file_size;
-	}
+	for_each_file(td, f, i)
+		f->real_file_size = -1ULL;
 
 	return 0;
 }
@@ -107,9 +102,9 @@ static int fio_null_init(struct thread_data *td)
 
 	memset(nd, 0, sizeof(*nd));
 
-	if (td->iodepth != 1) {
-		nd->io_us = malloc(td->iodepth * sizeof(struct io_u *));
-		memset(nd->io_us, 0, td->iodepth * sizeof(struct io_u *));
+	if (td->o.iodepth != 1) {
+		nd->io_us = malloc(td->o.iodepth * sizeof(struct io_u *));
+		memset(nd->io_us, 0, td->o.iodepth * sizeof(struct io_u *));
 	} else
 		td->io_ops->flags |= FIO_SYNCIO;
 
