@@ -65,15 +65,11 @@ static int pollin_events(struct pollfd *pfds, int fds)
 static int fio_sgio_getevents(struct thread_data *td, int min, int max,
 			      struct timespec fio_unused *t)
 {
-	/*
-	 * normally hard coding &td->files[0] is a bug that needs to be fixed,
-	 * but it's ok here as all files should point to the same device.
-	 */
-	struct fio_file *f = &td->files[0];
 	struct sgio_data *sd = td->io_ops->data;
 	int left = max, ret, r = 0;
 	void *buf = sd->sgbuf;
 	unsigned int i, events;
+	struct fio_file *f;
 
 	/*
 	 * Fill in the file descriptors
@@ -166,7 +162,7 @@ static int fio_sgio_ioctl_doio(struct thread_data *td,
 
 	ret = ioctl(f->fd, SG_IO, hdr);
 	if (ret < 0)
-		return -errno;
+		return ret;
 
 	return FIO_Q_COMPLETED;
 }
@@ -178,12 +174,12 @@ static int fio_sgio_rw_doio(struct fio_file *f, struct io_u *io_u, int sync)
 
 	ret = write(f->fd, hdr, sizeof(*hdr));
 	if (ret < 0)
-		return errno;
+		return ret;
 
 	if (sync) {
 		ret = read(f->fd, hdr, sizeof(*hdr));
 		if (ret < 0)
-			return -errno;
+			return ret;
 		return FIO_Q_COMPLETED;
 	}
 
@@ -390,7 +386,7 @@ static int fio_sgio_open(struct thread_data *td, struct fio_file *f)
 	if (ret)
 		return ret;
 
-	if (!sd->type_checked && fio_sgio_type_check(td, f)) {
+	if (sd && !sd->type_checked && fio_sgio_type_check(td, f)) {
 		generic_close_file(td, f);
 		return 1;
 	}
