@@ -76,7 +76,7 @@ static void terminate_threads(int group_id)
 			if (td->runstate < TD_RUNNING)
 				kill(td->pid, SIGQUIT);
 			td->terminate = 1;
-			td->start_delay = 0;
+			td->o.start_delay = 0;
 		}
 	}
 }
@@ -103,13 +103,14 @@ static void sig_handler(int sig)
 static int check_min_rate(struct thread_data *td, struct timeval *now)
 {
 	unsigned long long bytes = 0;
+	unsigned long iops = 0;
 	unsigned long spent;
 	unsigned long rate;
 
 	/*
 	 * No minimum rate set, always ok
 	 */
-	if (!td->ratemin)
+	if (!td->o.ratemin && !td->o.rate_iops_min)
 		return 0;
 
 	/*
@@ -118,41 +119,64 @@ static int check_min_rate(struct thread_data *td, struct timeval *now)
 	if (mtime_since(&td->start, now) < 2000)
 		return 0;
 
-	if (td_read(td))
+	if (td_read(td)) {
+		iops += td->io_blocks[DDIR_READ];
 		bytes += td->this_io_bytes[DDIR_READ];
-	if (td_write(td))
+	}
+	if (td_write(td)) {
+		iops += td->io_blocks[DDIR_WRITE];
 		bytes += td->this_io_bytes[DDIR_WRITE];
+	}
 
 	/*
 	 * if rate blocks is set, sample is running
 	 */
-	if (td->rate_bytes) {
+	if (td->rate_bytes || td->rate_blocks) {
 		spent = mtime_since(&td->lastrate, now);
-		if (spent < td->ratecycle)
+		if (spent < td->o.ratecycle)
 			return 0;
 
-		if (bytes < td->rate_bytes) {
-			fprintf(f_out, "%s: min rate %u not met\n", td->name, td->ratemin);
-			return 1;
-		} else {
-			rate = (bytes - td->rate_bytes) / spent;
-			if (rate < td->ratemin || bytes < td->rate_bytes) {
-				fprintf(f_out, "%s: min rate %u not met, got %luKiB/sec\n", td->name, td->ratemin, rate);
+		if (td->o.rate) {
+			/*
+			 * check bandwidth specified rate
+			 */
+			if (bytes < td->rate_bytes) {
+				log_err("%s: min rate %u not met\n", td->o.name, td->o.ratemin);
 				return 1;
+			} else {
+				rate = (bytes - td->rate_bytes) / spent;
+				if (rate < td->o.ratemin || bytes < td->rate_bytes) {
+					log_err("%s: min rate %u not met, got %luKiB/sec\n", td->o.name, td->o.ratemin, rate);
+					return 1;
+				}
+			}
+		} else {
+			/*
+			 * checks iops specified rate
+			 */
+			if (iops < td->o.rate_iops) {
+				log_err("%s: min iops rate %u not met\n", td->o.name, td->o.rate_iops);
+				return 1;
+			} else {
+				rate = (iops - td->rate_blocks) / spent;
+				if (rate < td->o.rate_iops_min || iops < td->rate_blocks) {
+					log_err("%s: min iops rate %u not met, got %lu\n", td->o.name, td->o.rate_iops_min, rate);
+				}
 			}
 		}
 	}
 
 	td->rate_bytes = bytes;
+	td->rate_blocks = iops;
 	memcpy(&td->lastrate, now, sizeof(*now));
 	return 0;
 }
 
 static inline int runtime_exceeded(struct thread_data *td, struct timeval *t)
 {
-	if (!td->timeout)
+	if (!td->o.timeout)
 		return 0;
-	if (mtime_since(&td->epoch, t) >= td->timeout * 1000)
+	if (mtime_since(&td->epoch, t) >= td->o.timeout * 1000)
 		return 1;
 
 	return 0;
@@ -318,10 +342,17 @@ static void do_verify(struct thread_data *td)
 					put_io_u(td, io_u);
 					break;
 				}
+
 				io_u->xfer_buflen = io_u->resid;
 				io_u->xfer_buf += bytes;
+				io_u->offset += bytes;
+
+				if (io_u->offset == io_u->file->real_file_size)
+					goto sync_done;
+
 				requeue_io_u(td, &io_u);
 			} else {
+sync_done:
 				ret = io_u_sync_complete(td, io_u);
 				if (ret < 0)
 					break;
@@ -352,8 +383,8 @@ static void do_verify(struct thread_data *td)
 		if (queue_full(td) || ret == FIO_Q_BUSY) {
 			min_events = 1;
 
-			if (td->cur_depth > td->iodepth_low)
-				min_events = td->cur_depth - td->iodepth_low;
+			if (td->cur_depth > td->o.iodepth_low)
+				min_events = td->cur_depth - td->o.iodepth_low;
 		}
 
 		/*
@@ -388,7 +419,7 @@ static void do_io(struct thread_data *td)
 
 	td_set_runstate(td, TD_RUNNING);
 
-	while ((td->this_io_bytes[0] + td->this_io_bytes[1]) < td->io_size) {
+	while ((td->this_io_bytes[0] + td->this_io_bytes[1]) < td->o.size) {
 		struct timeval comp_time;
 		long bytes_done = 0;
 		int min_evts = 0;
@@ -428,8 +459,14 @@ static void do_io(struct thread_data *td)
 
 				io_u->xfer_buflen = io_u->resid;
 				io_u->xfer_buf += bytes;
+				io_u->offset += bytes;
+
+				if (io_u->offset == io_u->file->real_file_size)
+					goto sync_done;
+
 				requeue_io_u(td, &io_u);
 			} else {
+sync_done:
 				fio_gettime(&comp_time, NULL);
 				bytes_done = io_u_sync_complete(td, io_u);
 				if (bytes_done < 0)
@@ -468,8 +505,8 @@ static void do_io(struct thread_data *td)
 			if (queue_full(td) || ret == FIO_Q_BUSY) {
 				min_evts = 1;
 
-				if (td->cur_depth > td->iodepth_low)
-					min_evts = td->cur_depth - td->iodepth_low;
+				if (td->cur_depth > td->o.iodepth_low)
+					min_evts = td->cur_depth - td->o.iodepth_low;
 			}
 
 			fio_gettime(&comp_time, NULL);
@@ -497,17 +534,17 @@ static void do_io(struct thread_data *td)
 			break;
 		}
 
-		if (td->thinktime) {
+		if (td->o.thinktime) {
 			unsigned long long b;
 
 			b = td->io_blocks[0] + td->io_blocks[1];
-			if (!(b % td->thinktime_blocks)) {
+			if (!(b % td->o.thinktime_blocks)) {
 				int left;
 
-				if (td->thinktime_spin)
-					__usec_sleep(td->thinktime_spin);
+				if (td->o.thinktime_spin)
+					__usec_sleep(td->o.thinktime_spin);
 
-				left = td->thinktime - td->thinktime_spin;
+				left = td->o.thinktime - td->o.thinktime_spin;
 				if (left)
 					usec_sleep(td, left);
 			}
@@ -521,7 +558,7 @@ static void do_io(struct thread_data *td)
 		if (i)
 			ret = io_u_queued_complete(td, i);
 
-		if (should_fsync(td) && td->end_fsync) {
+		if (should_fsync(td) && td->o.end_fsync) {
 			td_set_runstate(td, TD_FSYNCING);
 
 			for_each_file(td, f, i) {
@@ -564,6 +601,7 @@ static void fill_rand_buf(struct io_u *io_u, int max_bs)
 
 static int init_io_u(struct thread_data *td)
 {
+	unsigned long long buf_size;
 	struct io_u *io_u;
 	unsigned int max_bs;
 	int i, max_units;
@@ -572,15 +610,22 @@ static int init_io_u(struct thread_data *td)
 	if (td->io_ops->flags & FIO_SYNCIO)
 		max_units = 1;
 	else
-		max_units = td->iodepth;
+		max_units = td->o.iodepth;
 
-	max_bs = max(td->max_bs[DDIR_READ], td->max_bs[DDIR_WRITE]);
-	td->orig_buffer_size = max_bs * max_units;
+	max_bs = max(td->o.max_bs[DDIR_READ], td->o.max_bs[DDIR_WRITE]);
+	buf_size = (unsigned long long) max_bs * (unsigned long long) max_units;
+	buf_size += page_mask;
+	if (buf_size != (size_t) buf_size) {
+		log_err("fio: IO memory too large. Reduce max_bs or iodepth\n");
+		return 1;
+	}
 
-	if (td->mem_type == MEM_SHMHUGE || td->mem_type == MEM_MMAPHUGE)
-		td->orig_buffer_size = (td->orig_buffer_size + td->hugepage_size - 1) & ~(td->hugepage_size - 1);
-	else
-		td->orig_buffer_size += page_mask;
+	td->orig_buffer_size = buf_size;
+
+	if (td->o.mem_type == MEM_SHMHUGE || td->o.mem_type == MEM_MMAPHUGE)
+		td->orig_buffer_size = (td->orig_buffer_size + td->o.hugepage_size - 1) & ~(td->o.hugepage_size - 1);
+	else if (td->orig_buffer_size & page_mask)
+		td->orig_buffer_size = (td->orig_buffer_size + page_mask) & ~page_mask;
 
 	if (allocate_io_mem(td))
 		return 1;
@@ -625,7 +670,7 @@ static int switch_ioscheduler(struct thread_data *td)
 	/*
 	 * Set io scheduler.
 	 */
-	ret = fwrite(td->ioscheduler, strlen(td->ioscheduler), 1, f);
+	ret = fwrite(td->o.ioscheduler, strlen(td->o.ioscheduler), 1, f);
 	if (ferror(f) || ret != 1) {
 		td_verror(td, errno, "fwrite");
 		fclose(f);
@@ -644,9 +689,9 @@ static int switch_ioscheduler(struct thread_data *td)
 		return 1;
 	}
 
-	sprintf(tmp2, "[%s]", td->ioscheduler);
+	sprintf(tmp2, "[%s]", td->o.ioscheduler);
 	if (!strstr(tmp, tmp2)) {
-		log_err("fio: io scheduler %s not found\n", td->ioscheduler);
+		log_err("fio: io scheduler %s not found\n", td->o.ioscheduler);
 		td_verror(td, EINVAL, "iosched_switch");
 		fclose(f);
 		return 1;
@@ -666,6 +711,8 @@ static int clear_io_state(struct thread_data *td)
 	td->this_io_bytes[0] = td->this_io_bytes[1] = 0;
 	td->zone_bytes = 0;
 	td->rate_bytes = 0;
+	td->rate_blocks = 0;
+	td->rw_end_set[0] = td->rw_end_set[1] = 0;
 
 	td->last_was_sync = 0;
 
@@ -690,9 +737,10 @@ static void *thread_main(void *data)
 {
 	unsigned long long runtime[2];
 	struct thread_data *td = data;
+	unsigned long elapsed;
 	int clear_state;
 
-	if (!td->use_thread)
+	if (!td->o.use_thread)
 		setsid();
 
 	td->pid = getpid();
@@ -700,8 +748,9 @@ static void *thread_main(void *data)
 	INIT_LIST_HEAD(&td->io_u_freelist);
 	INIT_LIST_HEAD(&td->io_u_busylist);
 	INIT_LIST_HEAD(&td->io_u_requeues);
-	INIT_LIST_HEAD(&td->io_hist_list);
 	INIT_LIST_HEAD(&td->io_log_list);
+	INIT_LIST_HEAD(&td->io_hist_list);
+	td->io_hist_tree = RB_ROOT;
 
 	if (init_io_u(td))
 		goto err_sem;
@@ -721,15 +770,12 @@ static void *thread_main(void *data)
 		}
 	}
 
-	if (nice(td->nice) == -1) {
+	if (nice(td->o.nice) == -1) {
 		td_verror(td, errno, "nice");
 		goto err_sem;
 	}
 
-	if (init_random_state(td))
-		goto err_sem;
-
-	if (td->ioscheduler && switch_ioscheduler(td))
+	if (td->o.ioscheduler && switch_ioscheduler(td))
 		goto err_sem;
 
 	td_set_runstate(td, TD_INITIALIZED);
@@ -742,7 +788,7 @@ static void *thread_main(void *data)
 	 */
 	fio_sem_remove(td->mutex);
 
-	if (!td->create_serialize && setup_files(td))
+	if (!td->o.create_serialize && setup_files(td))
 		goto err;
 
 	if (td_io_init(td))
@@ -751,8 +797,11 @@ static void *thread_main(void *data)
 	if (open_files(td))
 		goto err;
 
-	if (td->exec_prerun) {
-		if (system(td->exec_prerun) < 0)
+	if (init_random_map(td))
+		goto err;
+
+	if (td->o.exec_prerun) {
+		if (system(td->o.exec_prerun) < 0)
 			goto err;
 	}
 
@@ -762,11 +811,11 @@ static void *thread_main(void *data)
 
 	runtime[0] = runtime[1] = 0;
 	clear_state = 0;
-	while (td->loops--) {
+	while (td->o.loops--) {
 		fio_gettime(&td->start, NULL);
 		memcpy(&td->ts.stat_sample_time, &td->start, sizeof(td->start));
 
-		if (td->ratemin)
+		if (td->o.ratemin)
 			memcpy(&td->lastrate, &td->ts.stat_sample_time, sizeof(td->lastrate));
 
 		if (clear_state && clear_io_state(td))
@@ -778,15 +827,27 @@ static void *thread_main(void *data)
 
 		clear_state = 1;
 
-		if (td_read(td) && td->io_bytes[DDIR_READ])
-			runtime[DDIR_READ] += utime_since_now(&td->start);
-		if (td_write(td) && td->io_bytes[DDIR_WRITE])
-			runtime[DDIR_WRITE] += utime_since_now(&td->start);
+		if (td_read(td) && td->io_bytes[DDIR_READ]) {
+			if (td->rw_end_set[DDIR_READ])
+				elapsed = utime_since(&td->start, &td->rw_end[DDIR_READ]);
+			else
+				elapsed = utime_since_now(&td->start);
+
+			runtime[DDIR_READ] += elapsed;
+		}
+		if (td_write(td) && td->io_bytes[DDIR_WRITE]) {
+			if (td->rw_end_set[DDIR_WRITE])
+				elapsed = utime_since(&td->start, &td->rw_end[DDIR_WRITE]);
+			else
+				elapsed = utime_since_now(&td->start);
+
+			runtime[DDIR_WRITE] += elapsed;
+		}
 		
 		if (td->error || td->terminate)
 			break;
 
-		if (td->verify == VERIFY_NONE)
+		if (td->o.verify == VERIFY_NONE)
 			continue;
 
 		if (clear_io_state(td))
@@ -815,11 +876,11 @@ static void *thread_main(void *data)
 		finish_log(td, td->ts.slat_log, "slat");
 	if (td->ts.clat_log)
 		finish_log(td, td->ts.clat_log, "clat");
-	if (td->write_iolog_file)
+	if (td->o.write_iolog_file)
 		write_iolog_close(td);
-	if (td->exec_postrun) {
-		if (system(td->exec_postrun) < 0)
-			log_err("fio: postrun %s failed\n", td->exec_postrun);
+	if (td->o.exec_postrun) {
+		if (system(td->o.exec_postrun) < 0)
+			log_err("fio: postrun %s failed\n", td->o.exec_postrun);
 	}
 
 	if (exitall_on_terminate)
@@ -831,6 +892,7 @@ err:
 	close_files(td);
 	close_ioengine(td);
 	cleanup_io_u(td);
+	options_mem_free(td);
 	td_set_runstate(td, TD_EXITED);
 	return (void *) (unsigned long) td->error;
 err_sem:
@@ -885,7 +947,7 @@ static void reap_threads(int *nr_running, int *t_rate, int *m_rate)
 
 		if (!td->pid || td->runstate == TD_REAPED)
 			continue;
-		if (td->use_thread) {
+		if (td->o.use_thread) {
 			if (td->runstate == TD_EXITED) {
 				td_set_runstate(td, TD_REAPED);
 				goto reaped;
@@ -931,7 +993,7 @@ static void reap_threads(int *nr_running, int *t_rate, int *m_rate)
 		 */
 		continue;
 reaped:
-		if (td->use_thread) {
+		if (td->o.use_thread) {
 			long ret;
 
 			if (pthread_join(td->thread, (void *) &ret))
@@ -939,8 +1001,8 @@ reaped:
 		}
 
 		(*nr_running)--;
-		(*m_rate) -= td->ratemin;
-		(*t_rate) -= td->rate;
+		(*m_rate) -= td->o.ratemin;
+		(*t_rate) -= td->o.rate;
 
 		if (td->error)
 			exit_value++;
@@ -986,7 +1048,7 @@ static void run_threads(void)
 	for_each_td(td, i) {
 		print_status_init(td->thread_number - 1);
 
-		if (!td->create_serialize) {
+		if (!td->o.create_serialize) {
 			init_disk_util(td);
 			continue;
 		}
@@ -1030,14 +1092,14 @@ static void run_threads(void)
 				continue;
 			}
 
-			if (td->start_delay) {
+			if (td->o.start_delay) {
 				spent = mtime_since_genesis();
 
-				if (td->start_delay * 1000 > spent)
+				if (td->o.start_delay * 1000 > spent)
 					continue;
 			}
 
-			if (td->stonewall && (nr_started || nr_running))
+			if (td->o.stonewall && (nr_started || nr_running))
 				break;
 
 			/*
@@ -1048,7 +1110,7 @@ static void run_threads(void)
 			map[this_jobs++] = td;
 			nr_started++;
 
-			if (td->use_thread) {
+			if (td->o.use_thread) {
 				if (pthread_create(&td->thread, NULL, thread_main, td)) {
 					perror("thread_create");
 					nr_started--;
@@ -1113,8 +1175,8 @@ static void run_threads(void)
 			td_set_runstate(td, TD_RUNNING);
 			nr_running++;
 			nr_started--;
-			m_rate += td->ratemin;
-			t_rate += td->rate;
+			m_rate += td->o.ratemin;
+			t_rate += td->o.rate;
 			todo--;
 			fio_sem_up(td->mutex);
 		}
