@@ -17,7 +17,7 @@
 #include "fio.h"
 #include "parse.h"
 
-static char fio_version_string[] = "fio 1.15";
+static char fio_version_string[] = "fio 1.15.1";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -545,10 +545,16 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 		p = fgets(string, 4095, f);
 		if (!p)
 			break;
+
+		strip_blank_front(&p);
+		strip_blank_end(p);
+
 		if (is_empty_or_comment(p))
 			continue;
-		if (sscanf(p, "[%255s]", name) != 1)
+		if (sscanf(p, "[%255s]", name) != 1) {
+			log_err("fio: option <%s> outside of job section\n", p);
 			continue;
+		}
 
 		global = !strncmp(name, "global", 6);
 
