@@ -215,7 +215,7 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 		int r;
 
 		td->io_u_queued++;
-		if (td->io_u_queued > td->iodepth_batch) {
+		if (td->io_u_queued > td->o.iodepth_batch) {
 			r = td_io_commit(td);
 			if (r < 0)
 				return r;
@@ -238,10 +238,15 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 
 int td_io_init(struct thread_data *td)
 {
-	if (td->io_ops->init)
-		return td->io_ops->init(td);
+	int ret = 0;
 
-	return 0;
+	if (td->io_ops->init) {
+		ret = td->io_ops->init(td);
+		if (ret && td->o.iodepth > 1)
+			log_err("fio: io engine init failed. Perhaps try reducing io dpeth?\n");
+	}
+
+	return ret;
 }
 
 int td_io_commit(struct thread_data *td)
@@ -258,8 +263,14 @@ int td_io_commit(struct thread_data *td)
 
 int td_io_open_file(struct thread_data *td, struct fio_file *f)
 {
-	if (td->io_ops->open_file(td, f))
+	if (td->io_ops->open_file(td, f)) {
+		if (td->error == EINVAL && td->o.odirect)
+			log_err("fio: destination does not support O_DIRECT\n");
+		if (td->error == EMFILE)
+			log_err("fio: try reducing/setting openfiles (failed at %u of %u)\n", td->nr_open_files, td->o.nr_files);
+
 		return 1;
+	}
 
 	f->last_free_lookup = 0;
 	f->last_completed_pos = 0;
@@ -267,12 +278,36 @@ int td_io_open_file(struct thread_data *td, struct fio_file *f)
 	f->flags |= FIO_FILE_OPEN;
 	f->flags &= ~FIO_FILE_CLOSING;
 
+	if (td->io_ops->flags & FIO_DISKLESSIO)
+		goto done;
+
+	if (td->o.invalidate_cache && file_invalidate_cache(td, f))
+		goto err;
+
+	if (td->o.fadvise_hint) {
+		int flags;
+
+		if (td_random(td))
+			flags = POSIX_FADV_RANDOM;
+		else
+			flags = POSIX_FADV_SEQUENTIAL;
+
+		if (fadvise(f->fd, f->file_offset, f->io_size, flags) < 0) {
+			td_verror(td, errno, "fadvise");
+			goto err;
+		}
+	}
+
 	if (f->file_map)
 		memset(f->file_map, 0, f->num_maps * sizeof(long));
 
+done:
 	td->nr_open_files++;
 	get_file(f);
 	return 0;
+err:
+	td->io_ops->close_file(td, f);
+	return 1;
 }
 
 void td_io_close_file(struct thread_data *td, struct fio_file *f)
