@@ -34,7 +34,7 @@ int fio_pin_memory(void)
 	if (phys_mem) {
 		if ((mlock_size + 128 * 1024 * 1024) > phys_mem) {
 			mlock_size = phys_mem - 128 * 1024 * 1024;
-			fprintf(f_out, "fio: limiting mlocked memory to %lluMiB\n", mlock_size >> 20);
+			log_info("fio: limiting mlocked memory to %lluMiB\n", mlock_size >> 20);
 		}
 	}
 
@@ -59,29 +59,33 @@ int fio_pin_memory(void)
  */
 int allocate_io_mem(struct thread_data *td)
 {
-	if (td->mem_type == MEM_MALLOC)
+	if (td->o.mem_type == MEM_MALLOC)
 		td->orig_buffer = malloc(td->orig_buffer_size);
-	else if (td->mem_type == MEM_SHM || td->mem_type == MEM_SHMHUGE) {
+	else if (td->o.mem_type == MEM_SHM || td->o.mem_type == MEM_SHMHUGE) {
 		int flags = IPC_CREAT | SHM_R | SHM_W;
 
-		if (td->mem_type == MEM_SHMHUGE)
+		if (td->o.mem_type == MEM_SHMHUGE)
 			flags |= SHM_HUGETLB;
 
 		td->shm_id = shmget(IPC_PRIVATE, td->orig_buffer_size, flags);
 		if (td->shm_id < 0) {
 			td_verror(td, errno, "shmget");
-			perror("shmget");
+			if (geteuid() != 0 && errno == ENOMEM)
+				log_err("fio: you may need to run this job as root\n");
+			if (errno == EINVAL && td->o.mem_type == MEM_SHMHUGE)
+				log_err("fio: check that you have free huge pages and that hugepage-size is correct.\n");
+			
 			return 1;
 		}
 
 		td->orig_buffer = shmat(td->shm_id, NULL, 0);
 		if (td->orig_buffer == (void *) -1) {
 			td_verror(td, errno, "shmat");
-			perror("shmat");
 			td->orig_buffer = NULL;
 			return 1;
 		}
-	} else if (td->mem_type == MEM_MMAP || td->mem_type == MEM_MMAPHUGE) {
+	} else if (td->o.mem_type == MEM_MMAP ||
+		   td->o.mem_type == MEM_MMAPHUGE) {
 		int flags = MAP_PRIVATE;
 
 		td->mmapfd = 0;
@@ -90,8 +94,12 @@ int allocate_io_mem(struct thread_data *td)
 			td->mmapfd = open(td->mmapfile, O_RDWR|O_CREAT, 0644);
 
 			if (td->mmapfd < 0) {
-				td_verror(td, errno, "open");
-				perror("open mmap file");
+				td_verror(td, errno, "open mmap file");
+				td->orig_buffer = NULL;
+				return 1;
+			}
+			if (ftruncate(td->mmapfd, td->orig_buffer_size) < 0) {
+				td_verror(td, errno, "truncate mmap file");
 				td->orig_buffer = NULL;
 				return 1;
 			}
@@ -101,7 +109,6 @@ int allocate_io_mem(struct thread_data *td)
 		td->orig_buffer = mmap(NULL, td->orig_buffer_size, PROT_READ | PROT_WRITE, flags, td->mmapfd, 0);
 		if (td->orig_buffer == MAP_FAILED) {
 			td_verror(td, errno, "mmap");
-			perror("mmap");
 			td->orig_buffer = NULL;
 			if (td->mmapfd) {
 				close(td->mmapfd);
@@ -117,14 +124,15 @@ int allocate_io_mem(struct thread_data *td)
 
 void free_io_mem(struct thread_data *td)
 {
-	if (td->mem_type == MEM_MALLOC)
+	if (td->o.mem_type == MEM_MALLOC)
 		free(td->orig_buffer);
-	else if (td->mem_type == MEM_SHM || td->mem_type == MEM_SHMHUGE) {
+	else if (td->o.mem_type == MEM_SHM || td->o.mem_type == MEM_SHMHUGE) {
 		struct shmid_ds sbuf;
 
 		shmdt(td->orig_buffer);
 		shmctl(td->shm_id, IPC_RMID, &sbuf);
-	} else if (td->mem_type == MEM_MMAP || td->mem_type == MEM_MMAPHUGE) {
+	} else if (td->o.mem_type == MEM_MMAP ||
+		   td->o.mem_type == MEM_MMAPHUGE) {
 		munmap(td->orig_buffer, td->orig_buffer_size);
 		if (td->mmapfile) {
 			close(td->mmapfd);
@@ -132,7 +140,7 @@ void free_io_mem(struct thread_data *td)
 			free(td->mmapfile);
 		}
 	} else
-		log_err("Bad memory type %u\n", td->mem_type);
+		log_err("Bad memory type %u\n", td->o.mem_type);
 
 	td->orig_buffer = NULL;
 }
