@@ -56,10 +56,4 @@
 #endif
 #endif
 
-#if __GNUC__ < 3
-#define __must_check
-#else
-#define __must_check			__attribute__((warn_unused_result))
-#endif
-
 #endif
