@@ -17,7 +17,7 @@ unsigned long utime_since(struct timeval *s, struct timeval *e)
 		usec += 1000000;
 	}
 
-	sec *= (double) 1000000;
+	sec *= 1000000UL;
 
 	return sec + usec;
 }
@@ -41,8 +41,8 @@ unsigned long mtime_since(struct timeval *s, struct timeval *e)
 		usec += 1000000;
 	}
 
-	sec *= (double) 1000;
-	usec /= (double) 1000;
+	sec *= 1000UL;
+	usec /= 1000UL;
 
 	return sec + usec;
 }
@@ -114,15 +114,15 @@ void rate_throttle(struct thread_data *td, unsigned long time_spent,
 	unsigned long usec_cycle;
 	unsigned int bs;
 
-	if (!td->rate)
+	if (!td->o.rate && !td->o.rate_iops)
 		return;
 
 	if (td_rw(td))
-		bs = td->rw_min_bs;
+		bs = td->o.rw_min_bs;
 	else if (td_read(td))
-		bs = td->min_bs[DDIR_READ];
+		bs = td->o.min_bs[DDIR_READ];
 	else
-		bs = td->min_bs[DDIR_WRITE];
+		bs = td->o.min_bs[DDIR_WRITE];
 
 	usec_cycle = td->rate_usec_cycle * (bytes / bs);
 
@@ -130,9 +130,13 @@ void rate_throttle(struct thread_data *td, unsigned long time_spent,
 		unsigned long s = usec_cycle - time_spent;
 
 		td->rate_pending_usleep += s;
+
 		if (td->rate_pending_usleep >= 100000) {
+			struct timeval t;
+
+			fio_gettime(&t, NULL);
 			usec_sleep(td, td->rate_pending_usleep);
-			td->rate_pending_usleep = 0;
+			td->rate_pending_usleep -= utime_since_now(&t);
 		}
 	} else {
 		long overtime = time_spent - usec_cycle;
