@@ -38,8 +38,8 @@ static void hexdump(void *buffer, int len)
 	int i;
 
 	for (i = 0; i < len; i++)
-		fprintf(f_out, "%02x", p[i]);
-	fprintf(f_out, "\n");
+		log_info("%02x", p[i]);
+	log_info("\n");
 }
 
 static int verify_io_u_crc32(struct verify_header *hdr, struct io_u *io_u)
@@ -78,11 +78,14 @@ static int verify_io_u_md5(struct verify_header *hdr, struct io_u *io_u)
 	return 0;
 }
 
-int verify_io_u(struct io_u *io_u)
+int verify_io_u(struct thread_data *td, struct io_u *io_u)
 {
 	struct verify_header *hdr = (struct verify_header *) io_u->buf;
 	int ret;
 
+	if (td->o.verify == VERIFY_NULL)
+		return 0;
+
 	if (hdr->fio_magic != FIO_HDR_MAGIC) {
 		log_err("Bad verify header %x\n", hdr->fio_magic);
 		return EIO;
@@ -126,15 +129,18 @@ void populate_verify_io_u(struct thread_data *td, struct io_u *io_u)
 	unsigned char *p = (unsigned char *) io_u->buf;
 	struct verify_header hdr;
 
+	if (td->o.verify == VERIFY_NULL)
+		return;
+
 	hdr.fio_magic = FIO_HDR_MAGIC;
 	hdr.len = io_u->buflen;
 	p += sizeof(hdr);
 	fill_random_bytes(td, p, io_u->buflen - sizeof(hdr));
 
-	if (td->verify == VERIFY_MD5) {
+	if (td->o.verify == VERIFY_MD5) {
 		fill_md5(&hdr, p, io_u->buflen - sizeof(hdr));
 		hdr.verify_type = VERIFY_MD5;
-	} else {
+	} else if (td->o.verify == VERIFY_CRC32) {
 		fill_crc32(&hdr, p, io_u->buflen - sizeof(hdr));
 		hdr.verify_type = VERIFY_CRC32;
 	}
@@ -144,7 +150,7 @@ void populate_verify_io_u(struct thread_data *td, struct io_u *io_u)
 
 int get_next_verify(struct thread_data *td, struct io_u *io_u)
 {
-	struct io_piece *ipo;
+	struct io_piece *ipo = NULL;
 
 	/*
 	 * this io_u is from a requeue, we already filled the offsets
@@ -152,11 +158,17 @@ int get_next_verify(struct thread_data *td, struct io_u *io_u)
 	if (io_u->file)
 		return 0;
 
-	if (!list_empty(&td->io_hist_list)) {
+	if (!RB_EMPTY_ROOT(&td->io_hist_tree)) {
+		struct rb_node *n = rb_first(&td->io_hist_tree);
+
+		ipo = rb_entry(n, struct io_piece, rb_node);
+		rb_erase(n, &td->io_hist_tree);
+	} else if (!list_empty(&td->io_hist_list)) {
 		ipo = list_entry(td->io_hist_list.next, struct io_piece, list);
-
 		list_del(&ipo->list);
+	}
 
+	if (ipo) {
 		io_u->offset = ipo->offset;
 		io_u->buflen = ipo->len;
 		io_u->file = ipo->file;
